#include "moduleQuestion2et3.h"

/* -------------------------------------------------------------------- */
/* fctContientMotif, permet de détecter si une chaine contient un       */
/*                   certains motif                                     */
/*                                                                      */
/* En entrée:   chaine, pointeur vers la chaine à analyser              */
/*              motif, pointeur vers la chaine à détecter               */
/*                                                                      */
/* En sortie: 1 si le motif et présent, 0 sinon                         */
/* -------------------------------------------------------------------- */

int fctContientMotif(char *chaine, char *motif) {

  int OK = 0;
  int compteurChaine = 0;
  int compteurMotif = 0;

  if (*motif == '\0') {
    OK = 1;
  } else {

    while ((OK == 0) && (*(chaine + compteurChaine) != '\0')) {
      if (*(chaine + compteurChaine) == *motif) {

        compteurMotif = 0;
        while ((*(chaine + compteurChaine + compteurMotif) ==
                *(motif + compteurMotif)) &&
               (*(chaine + compteurChaine + compteurMotif) != '\0') &&
               (OK == 0)) {
          compteurMotif++;
          if (*(motif + compteurMotif) == '\0') {
            OK = 1;
          }
        }
      }
      compteurChaine++;
    }
  }

  return OK;
}

/* -------------------------------------------------------------------- */
/* remplirStructureListeC_AvecFicTxt, permet de creer la liste contigue */
/* contenant les éléments ayant le motif requis                         */
/*                                                                      */
/* En entrée:   listeC est un pointeur de la list contigüe              */
/*              nomFichier est le nom du fichier à analyser             */
/*              motif est les otif à chercher dans le fichier           */
/*              listeContigue est la liste qui va contenir les éléments */
/* du fichier                                                           */
/*                                                                      */
/* En sortie: Aucune sortie                                             */
/* -------------------------------------------------------------------- */

void remplirStructureListeC_AvecFicTxt(listeC_t *listeC, char *nomFichier,
                                       char *motif,
                                       char listeContigue[][TAILLE_MAX]) {

  FILE *fichier = NULL;
  char chaine[TAILLE_MAX];
  fichier = fopen(nomFichier, "r");

  int compteurListeC = 0;

  if (fichier != NULL) {
    strcpy(chaine, "\0");
    while (fgets(chaine, TAILLE_MAX, fichier) != NULL) {
      if (fctContientMotif(chaine + 9, motif) == 1) {

        if (compteurListeC == 0) {
          strcpy(listeContigue[compteurListeC], chaine);
          compteurListeC++;
        } else {
          if (strcmp(chaine, listeContigue[compteurListeC - 1]) != 0) {
            strcpy(listeContigue[compteurListeC], chaine);
            compteurListeC++;
          }
        }
      }
      strcpy(chaine, "\0");
    }
    fclose(fichier);
  }

  listeC->debut = *listeContigue;
  listeC->fin = *(listeContigue + compteurListeC - 1);
}

/* -------------------------------------------------------------------- */
/* creerTempFiltre, permet de creer un fichier temp contenant les       */
/* lignes souhaités, en fonction du mode, les lignes étant selectionnées*/
/* à l'aide de la date                                                  */
/*                                                                      */
/* En entrée:   nomFichier, nom du fichier ayant les infos a traiter    */
/*              NomFichierTemp, nom du fichier temp pour le programme   */
/*                              permettant de l'utiliser pour les autre */
/*                              fonction du prgramme                    */
/*              motif, motif recherché                                  */
/*              annee, année sélectionnée                               */
/*              semaine, semaine sélectionnée                           */
/*              jour, jour sélectionnée                                 */
/*              heure, heure sélectionnée                               */
/*              mode, choix du mode, SELECT, qui garde les lignes       */
/*                    sélectionnée, SUPPR qui supprime celles-ci        */
/*                                                                      */
/* En sortie: Aucune sortie                                             */
/* -------------------------------------------------------------------- */

void creerTempFiltre(char nomFichier[TAILLE_MAX],
                     char nomFichierTemp[TAILLE_MAX], char motif[TAILLE_MAX],
                     char annee[5], char semaine[3], char jour[2],
                     char heure[3], int mode) {

  FILE *fichier = NULL;
  fichier = fopen(nomFichier, "r");

  FILE *temp = NULL;
  temp = fopen(nomFichierTemp, "w");

  char chaine[TAILLE_MAX];
  char chaineTemp[TAILLE_MAX];

  if (fichier != NULL) {
    while (fgets(chaine, TAILLE_MAX, fichier) != NULL) {
      int OK = 1;

      if (fctContientMotif(chaine + 9, motif) == mode) {
        if (motif[0] != '\0') {
          OK = 0;
        }
      }

      strcpy(chaineTemp, chaine);
      chaineTemp[4] = '\0';
      if (fctContientMotif(chaineTemp, annee) == mode) {
        if (annee[0] != '\0') {
          OK = 0;
        }
      }

      strcpy(chaineTemp, chaine);
      chaineTemp[6] = '\0';
      if (fctContientMotif(chaineTemp + 4, semaine) == mode) {
        if (semaine[0] != '\0') {
          OK = 0;
        }
      }

      strcpy(chaineTemp, chaine);
      chaineTemp[7] = '\0';
      if (fctContientMotif(chaineTemp + 6, jour) == mode) {
        if (jour[0] != '\0') {
          OK = 0;
        }
      }

      strcpy(chaineTemp, chaine);
      chaineTemp[9] = '\0';
      if (fctContientMotif(chaineTemp + 7, heure) == mode) {
        if (heure[0] != '\0') {
          OK = 0;
        }
      }

      if (OK == 1) {
        fprintf(temp, "%s", chaine);
      }
    }
    fclose(fichier);
  }
  fclose(temp);
}