#include "moduleBase.h"

/* -------------------------------------------------------------------- */
/* tailleFichier, permet de connaitre le nombre de lignes d'un fichier  */
/*                                                                      */
/* En entrée:    le nom du fichier dont il faut compter les lignes      */
/*                                                                      */
/* En sortie: le nombre de lignes du fichier                            */
/* -------------------------------------------------------------------- */

int tailleFichier(char nomFichier[TAILLE_MAX]) {

  FILE *fichier = NULL; // on ouvre le fichier en mode lecture
  char chaine[TAILLE_MAX];
  fichier = fopen(nomFichier, "r");

  int taille = 0;

  if (fichier != NULL) { // si le fichier existe bien
    while (fgets(chaine, TAILLE_MAX, fichier) !=
           NULL) { // tant que on est pas arrivés au bout du fichier
      taille++;
    }
    fclose(fichier);
  }
  return taille;
}

/* -------------------------------------------------------------------- */
/* creerTete permet de créer la tête de la liste                        */
/*                                                                      */
/* En entrée: Aucune entrée                                             */
/*                                                                      */
/* En sortie: ptrTete, le pointeur de la tête de la liste               */
/* -------------------------------------------------------------------- */

semaine_t *creerTete() {

  semaine_t *ptrTete = malloc(sizeof(semaine_t)); // on crée et on initialise la tete avant de la retourner

  strcpy(ptrTete->annee, "\0");
  strcpy(ptrTete->semaine, "\0");
  ptrTete->act = NULL;
  ptrTete->suiv = NULL;

  return ptrTete;
}

/* -------------------------------------------------------------------- */
/* creerTab, permet de créer le tableau contenant les lignes du fichier */
/*                                                                      */
/* En entrée:   tabPhrase, va accueillir les lignes du fichier en vue de les trier temporellement avec atoi()                                        */
/*              nomFichier, le nom du fichier dont on veut récupèrer les lignes                                       */
/*                                                                      */
/* En sortie: Aucune sortie                                             */
/* -------------------------------------------------------------------- */

void creerTab(char tabPhrases[][TAILLE_MAX], char *nomFichier) {
  
  FILE *fichier = NULL;
  fichier = fopen(nomFichier, "r");
  int compteur = 0;
  char chaine[TAILLE_MAX];
  int compteurChaine=0;

  strcpy(tabPhrases[0],"fichier vide !");

  if (fichier != NULL) {
    while (fgets(chaine, TAILLE_MAX, fichier) != NULL) {
      compteurChaine=0;
      while(chaine[compteurChaine] != '\0'){
        if(chaine[compteurChaine] == '\n'){
          chaine[compteurChaine]='\0';
        }
        compteurChaine++;
      }
      strcpy(tabPhrases[compteur],chaine);
      compteur++;
    }
    fclose(fichier);
  }
}

/* -------------------------------------------------------------------- */
/* triFusion, effectue un tri fusion sur un tableau, fonction récursive */
/*                                                                      */
/* En entrée:   i j, les indices de début et de fin du tableau          */
/*              tabPhrase, le tableau qui se fait trier                 */
/*              tmp, sert à la fonction de tri                          */
/*                                                                      */
/* En sortie: Aucune sortie                                             */
/* -------------------------------------------------------------------- */

void triFusion(int i, int j, char tabPhrases[][TAILLE_MAX],
               char tmp[][TAILLE_MAX]) {

  if (j <= i) {
    return;
  }

  int m = (i + j) / 2;

  triFusion(i, m, tabPhrases, tmp); // trier la moitié gauche récursivement
  triFusion(m + 1, j, tabPhrases, tmp); // trier la moitié droite récursivement

  int pg = i;     // pg pointe au début du sous-tableau de gauche
  int pd = m + 1; // pd pointe au début du sous-tableau de droite
  int c;          // compteur

  // on boucle de i à j pour remplir chaque élément du tableau final fusionné
  for (c = i; c <= j; c++) {
    if (pg ==
        m + 1) { // le pointeur du sous-tableau de gauche a atteint la limite
      strcpy(tmp[c], tabPhrases[pd]);
      pd++;
    } else if (pd == j + 1) { // le pointeur du sous-tableau de droite a atteint
                              // la limite
      strcpy(tmp[c], tabPhrases[pg]);
      pg++;
    } else if (atoi(tabPhrases[pg]) <
               atoi(tabPhrases[pd])) { // le pointeur du sous-tableau de gauche
                                       // pointe vers un élément plus petit
      strcpy(tmp[c], tabPhrases[pg]);
      pg++;
    } else { // le pointeur du sous-tableau de droite pointe vers un élément
             // plus petit
      strcpy(tmp[c], tabPhrases[pd]);
      pd++;
    }
  }

  for (c = i; c <= j; c++) { // copier les éléments de tmp[] à tab[]
    strcpy(tabPhrases[c], tmp[c]);
  }
}

/* -------------------------------------------------------------------- */
/* remplirTab, va séparé les différents éléments contenu dans chaque    */
/*             ligne de notre tableau                                   */
/*                                                                      */
/* En entrée:   tabPhraseTriees                                         */
/*              tab                                                     */
/*              taille                                                  */
/*                                                                      */
/* En sortie: Aucune sortie                                             */
/* -------------------------------------------------------------------- */

void remplirTab(char tabPhrasesTriees[][TAILLE_MAX], char tab[][5][TAILLE_MAX],
                int taille) {

  int compteur = 0;
  for (int u = 0; u < taille; u++) {
    // while(strcmp(tabPhrasesTriees[compteur],"\0") != 0){
    tab[compteur][0][4] = '\0';
    tab[compteur][1][2] = '\0';
    tab[compteur][2][1] = '\0';
    tab[compteur][3][2] = '\0';
    tab[compteur][4][10] = '\0';

    for (int i = 0; i < 4; i++) {
      tab[compteur][0][i] = tabPhrasesTriees[compteur][i];
    }
    for (int i = 0; i < 2; i++) {
      tab[compteur][1][i] = tabPhrasesTriees[compteur][4 + i];
    }
    tab[compteur][2][0] = tabPhrasesTriees[compteur][6];
    for (int i = 0; i < 2; i++) {
      tab[compteur][3][i] = tabPhrasesTriees[compteur][7 + i];
    }
    for (int i = 0; i < 10; i++) {
      tab[compteur][4][i] = tabPhrasesTriees[compteur][9 + i];
    }
    compteur++;
  }
}

/* -------------------------------------------------------------------- */
/* creerListChainee, permet de créer les listes chainées               */
/*                                                                      */
/* En entrée:    ptrSemaineTete, pointeur de la tête de la liste        */
/*               taille, nombre de ligne du tableau tab                 */
/*               tab, tableau des lignes de l'agenda traitée (mise en   */
/*               page + tri)                                            */
/*                                                                      */
/* En sortie: Aucune sortie                                             */
/* -------------------------------------------------------------------- */

void creerListeChainee(semaine_t *ptrSemaineTete, char tab[][5][TAILLE_MAX],
                       int taille) {

  int compteur;
  for (compteur = 0; compteur < taille; compteur++) {
    semaine_t *ptrSemaineCour = ptrSemaineTete;
    while (ptrSemaineCour->suiv != NULL) {
      ptrSemaineCour = ptrSemaineCour->suiv;
    }

    if ((atoi(ptrSemaineCour->annee) == atoi(tab[compteur][0])) &&
        (atoi(ptrSemaineCour->semaine) == atoi(tab[compteur][1]))) {

      action_t *ptrNouvelleAction = malloc(sizeof(action_t));
      strcpy(ptrNouvelleAction->jour, tab[compteur][2]);
      strcpy(ptrNouvelleAction->heure, tab[compteur][3]);
      strcpy(ptrNouvelleAction->nom, tab[compteur][4]);
      ptrNouvelleAction->suiv = NULL;
     
      action_t *ptrActionCour = ptrSemaineCour->act;
      if (ptrActionCour == NULL) {
        ptrSemaineCour->act = ptrNouvelleAction;
        
      } else {
        while (ptrActionCour->suiv != NULL) {
          ptrActionCour = ptrActionCour->suiv;
        }

        ptrActionCour->suiv = ptrNouvelleAction;
       
      }
    } else {
      

      semaine_t *ptrNouvelleSemaine = malloc(sizeof(semaine_t));
      strcpy(ptrNouvelleSemaine->annee, tab[compteur][0]);
      strcpy(ptrNouvelleSemaine->semaine, tab[compteur][1]);
      ptrNouvelleSemaine->act = malloc(sizeof(action_t));
     
      ptrNouvelleSemaine->suiv = NULL;
     
      strcpy(ptrNouvelleSemaine->act->jour, tab[compteur][2]);
      strcpy(ptrNouvelleSemaine->act->heure, tab[compteur][3]);
      strcpy(ptrNouvelleSemaine->act->nom, tab[compteur][4]);
      ptrNouvelleSemaine->act->suiv = NULL;
      ptrSemaineCour->suiv = ptrNouvelleSemaine;
    }
  }
}

/* -------------------------------------------------------------------- */
/* afficherAgenda, permet d'afficher l'agenda dans le terminal          */
/*                                                                      */
/* En entrée:    ptrSemaineTete, pointeur de la tête de la liste        */
/*               taille                                                 */
/*                                                                      */
/* En sortie: Aucune sortie, uniquement affichage de l'agenda           */
/* -------------------------------------------------------------------- */

void afficherAgenda(struct semaine *ptrTeteReelle) {
  action_t *ptrAction;
  semaine_t *ptrSemCour = ptrTeteReelle->suiv;
  if (ptrSemCour == NULL) {
    printf("agenda vide\n");
  } else {
    while (ptrSemCour != NULL) {
      printf("annee : %s  semaine : %s\n", ptrSemCour->annee,
             ptrSemCour->semaine);
      ptrAction = ptrSemCour->act;
      while (ptrAction != NULL) {
        printf("            jour : %s   heure : %s     nom : %s\n",
               ptrAction->jour, ptrAction->heure, ptrAction->nom);
        ptrAction = ptrAction->suiv;
      }
      ptrSemCour = ptrSemCour->suiv;
    }
  }
}

/* -------------------------------------------------------------------- */
/* SDDintoFILE permet d'écrire notre nouvel agenda dans un fichier      */
/*                                                                      */
/* En entrée: ptrTeteRelle, pointeur de la tête du 1er niveau de la     */
/* liste chainée                                                        */
/*                                                                      */
/* En sortie: Aucune sortie                                             */
/* -------------------------------------------------------------------- */

void SDDintoFILE(semaine_t *ptrTeteReelle) {

  FILE *fichier2 = NULL;
  char chaine[TAILLE_MAX];
  fichier2 = fopen("resultatCommande.txt", "w");
  
  int init = 0;

  semaine_t *ptrSemCour = ptrTeteReelle;
  action_t *ptrActCour;

  while (ptrSemCour != NULL) {
    ptrActCour = ptrSemCour->act;
    while (ptrActCour != NULL) {
      strcpy(chaine, ptrSemCour->annee);
      strcat(chaine, ptrSemCour->semaine);
      strcat(chaine, ptrActCour->jour);
      strcat(chaine, ptrActCour->heure);
      strcat(chaine, ptrActCour->nom);
      if (init == 0) {
        fprintf(fichier2, "%s", chaine);
        init = 1;
      } else {
        fprintf(fichier2, "\n%s", chaine);
      }

      ptrActCour = ptrActCour->suiv;
    }
    ptrSemCour = ptrSemCour->suiv;
  }
  fclose(fichier2);
}

/* -------------------------------------------------------------------- */
/* freeall permet de libérer les allocations mémoire                    */
/*                                                                      */
/* En entrée: ptrTeteRelle, pointeur de la tête du 1er niveau de la     */
/* liste chainée                                                        */
/*                                                                      */
/* En sortie: Aucune sortie                                             */
/* -------------------------------------------------------------------- */

void freeAll(semaine_t *ptrTeteReelle) {
  semaine_t *ptrSemPrecFree = ptrTeteReelle;
  semaine_t *ptrSemCourFree = ptrTeteReelle->suiv;

  while (ptrSemCourFree != NULL) {

    action_t *ptrActionPrecFree = ptrSemCourFree->act;
    action_t *ptrActionCourFree = ptrActionPrecFree->suiv;

    while (ptrActionCourFree != NULL) {
      free(ptrActionPrecFree);
      ptrActionPrecFree = ptrActionCourFree;
      ptrActionCourFree = ptrActionCourFree->suiv;
    }
    free(ptrActionPrecFree);
    free(ptrSemPrecFree);
    ptrSemPrecFree = ptrSemCourFree;
    ptrSemCourFree = ptrSemCourFree->suiv;
  }
  free(ptrSemPrecFree);
}

/* -------------------------------------------------------------------- */
/* creerSDDafficherAgendaPrintRecap permet de mettre en relation les    */
/* fonctions ci-dessus afin d'afficher et d'ecrire correctement la liste*/
/* chainée recue                                                        */
/*                                                                      */
/* En entrée: ptrTeteRelle, pointeur de la tête du 1er niveau de la     */
/* liste chainée                                                        */
/*                                                                      */
/* En sortie: Aucune sortie                                             */
/* -------------------------------------------------------------------- */

void creerSDDafficherAgendaPrintRecap(char nomFichier[TAILLE_MAX]) {

  int taille = tailleFichier(nomFichier);
  char tabPhrases[taille][TAILLE_MAX];
  char tmp[taille][TAILLE_MAX];
  char tab[taille][5][TAILLE_MAX];
  semaine_t *ptrSemaineTete = NULL;

  ptrSemaineTete = creerTete();

  creerTab(tabPhrases, nomFichier);
  triFusion(0, taille - 1, tabPhrases, tmp);
  remplirTab(tabPhrases, tab, taille);
  creerListeChainee(ptrSemaineTete, tab, taille);
  afficherAgenda(ptrSemaineTete);
  SDDintoFILE(ptrSemaineTete);
  freeAll(ptrSemaineTete);
}