#include "teZZt.h"

static unsigned __tezzt__passed_tests_counter = 0;
static unsigned __tezzt__failed_tests_counter = 0;

void __tezzt__passed_test() { ++__tezzt__passed_tests_counter; }
void __tezzt__failed_test() { ++__tezzt__failed_tests_counter; }

void __tezzt__exit_program_with_report() {
  fprintf(stderr, "--- teZZt REPORT ---\n");
  fprintf(stderr, __ANSI_RED__ "%3d test(s) failed\n",
          __tezzt__failed_tests_counter);
  fprintf(stderr, __ANSI_GRE__ "%3d test(s) passed\n",
          __tezzt__passed_tests_counter);
  fprintf(stderr, __ANSI_NOR__);
}

BEGIN_TEST_GROUP(tableauStructure)

TEST(tailleFichier) {

  CHECK(9 == tailleFichier("test1.txt"));
  CHECK(12 == tailleFichier("test2.txt"));
  CHECK(0 == tailleFichier("test3.txt"));
  
}

TEST(creerTete){

  semaine_t *ptrTeteReelle = creerTete();
  
  CHECK(0 == strcmp(ptrTeteReelle->annee, "\0"));
  CHECK(0 == strcmp(ptrTeteReelle->semaine, "\0"));
  CHECK(NULL == ptrTeteReelle->act);
  CHECK(NULL == ptrTeteReelle->suiv);
  
  free(ptrTeteReelle);
  
}

TEST(creerTab){

  char tab[100][TAILLE_MAX];
  creerTab(tab, "test1.txt");

  CHECK(0 == strcmp(tab[0], "202215108TPs de SDD"));
  CHECK(0 == strcmp(tab[8], "201951421re piscine"));

  creerTab(tab, "test2.txt");

  CHECK(0 == strcmp(tab[0], "202215108TPs de SDD"));
  CHECK(0 == strcmp(tab[11], "202215109test num 6"));

  creerTab(tab, "test3.txt");

  CHECK(0 == strcmp(tab[0], "fichier vide !"));
  
}

TEST(triFusion){

  char tabASCII[15][TAILLE_MAX]={"215ze5f","789vv","1234tgh56","987","456zef56","0","1","2"};
  char tmp[15][TAILLE_MAX];
  
  triFusion(0,7,tabASCII,tmp);

  CHECK(0 == strcmp(tabASCII[0], "0"));
  CHECK(0 == strcmp(tabASCII[1], "1"));
  CHECK(0 == strcmp(tabASCII[6], "987"));
  CHECK(0 == strcmp(tabASCII[7], "1234tgh56"));
  

  
  char tmp2[15][TAILLE_MAX];
  char tabPhrases[15][TAILLE_MAX];
  creerTab(tabPhrases, "test1.txt");
  triFusion(0,8,tabPhrases,tmp2);

  CHECK(0 == strcmp(tabPhrases[0], "201951420go piscine"));
  CHECK(0 == strcmp(tabPhrases[1], "201951421re piscine"));
  CHECK(0 == strcmp(tabPhrases[6], "202640322test num 6"));
  CHECK(0 == strcmp(tabPhrases[8], "202640323test num 4"));
  

}


TEST(SDDintoFile){

  semaine_t * ptrTete=creerTete();
  semaine_t * ptrSemaine1=malloc(sizeof(semaine_t));
  strcpy(ptrSemaine1->annee,"2020");
  strcpy(ptrSemaine1->semaine,"10");
  ptrSemaine1->act = malloc(sizeof(action_t));
  strcpy(ptrSemaine1->act->jour,"2");
  strcpy(ptrSemaine1->act->heure,"22");
  strcpy(ptrSemaine1->act->nom,"act1deTest");
  ptrSemaine1->act->suiv=malloc(sizeof(action_t));
  strcpy(ptrSemaine1->act->suiv->jour,"3");
  strcpy(ptrSemaine1->act->suiv->heure,"19");
  strcpy(ptrSemaine1->act->suiv->nom,"act2deTest");
  ptrSemaine1->act->suiv->suiv=NULL;
  ptrSemaine1->suiv = NULL;


  ptrTete->suiv=ptrSemaine1;
  
  SDDintoFILE(ptrTete);

  FILE * fichier=fopen("resultatCommande.txt","r");
  char chaine[2][TAILLE_MAX];
  int compteur=0;
  if( fichier != NULL){
    while(fgets(chaine[compteur],TAILLE_MAX,fichier) != NULL){
      compteur++;
    }
    fclose(fichier);
  }
  chaine[0][19]='\0'; // on enlève le saut de ligne

  CHECK(0 == strcmp(chaine[0], "202010222act1deTest"));
  CHECK(0 == strcmp(chaine[1], "202010319act2deTest"));
  
}

TEST(fctContientMotif){

CHECK(1 == fctContientMotif("idgdmotifqsdygzyie","motif"));
CHECK(0 == fctContientMotif("idgd_pasmotif_qsdygzyie","pasMotif"));
CHECK(0 == fctContientMotif("idgd_motif","motifTropLong0123456"));
  
}
// 202215108TPs de SDD
// 202326415Vacs ete  
// 202240323test_num_3
// 202140323test num 1
// 202640323test num 4
// 202640322test num 5
// 202640322test num 6
// 201951420go piscine
// 201951421re piscine

 


END_TEST_GROUP(tableauStructure)
